!******************************************************************************!
!                                                                              !
!    lcVFFa.f90                                                                !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2018 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcVFFa.f90                                     |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 07/2017                              |            !
!        |       | UFO version: 1.0                               |            !
!        |       | Generation date: 2020-07-29 14:45             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcVFFa
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29
  contains
    subroutine loopVFFa(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,      &
    riMaxOut, rankInc)
      use class_vertices
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(VFFa0)
!       GammaP(1,3,2)*c0
!       GammaM(1,3,2)*c1
        pl(:) = PL1(:) + PL2(:)
        x22 = pl(3)*w2(0)
        x23 = pl(1)*w2(0)
        x24 = cima*co(0)
        x25 = co(1)*m
        x26 = pl(1)*w2(3)
        x27 = pl(2)*w2(3)
        do j = riMaxIn, 0, -1
          x0 = w2(1)*wl(2,j)
          x1 = w2(1)*wl(1,j)
          x2 = wl(0,j) - wl(3,j)
          x3 = w2(0)*x2
          x4 = co(0)*(cima*x1 - cima*x3 + x0)
          x5 = wl(0,j) + wl(3,j)
          x6 = w2(1)*x5
          x7 = cima*wl(1,j)
          x8 = co(0)*(cima*x6 + w2(0)*wl(2,j) - w2(0)*x7)
          x9 = w2(2)*x5
          x10 = co(1)*(cima*x9 + w2(3)*wl(2,j) + w2(3)*x7)
          x11 = w2(2)*wl(2,j)
          x12 = -x11
          x13 = w2(2)*wl(1,j)
          x14 = w2(3)*x2
          x15 = co(1)*(cima*x13 + cima*x14 + x12)
          x16 = cima*wl(2,j)
          x17 = wl(1,j) + x16
          x18 = -x1
          x19 = x13 + x14
          x20 = cima*x19
          x21 = w2(0)*(-wl(0,j) + wl(3,j)) + x1
          wp(0,j) = -co(0)*(cima*pl(0)*x21 - cima*pl(3)*x6 + cima*wl(1,j)*x22  &
                     + pl(0)*w2(1)*wl(2,j) - wl(2,j)*x22) - m*x10
          wp(1,j) = co(0)*wl(2,j)*(-pl(2)*w2(1) + x23) + pl(1)*x24*x6 - pl(2)* &
                    x21*x24 - wl(1,j)*x23*x24 + x11*x25 - x20*x25
          wp(2,j) = cima*co(1)*(pl(1)*x9 - pl(3)*x19 + wl(1,j)*x26) + co(0)*m* &
                    x0 + co(1)*wl(2,j)*(pl(3)*w2(2) + x26) - m*x24*(x18 + x3)
          wp(3,j) = -co(1)*(-cima*pl(0)*x19 + cima*pl(2)*x9 + cima*wl(1,j)*x27 &
                     + pl(0)*w2(2)*wl(2,j) + wl(2,j)*x27) - m*x8
          if(rankInc .lt. 1) cycle
          sp2(0,0) = -x4
          sp2(0,1) = x8
          sp2(0,2) = x10
          sp2(0,3) = x15
          sp2(1,0) = -x8
          sp2(1,1) = x4
          sp2(1,2) = x15
          sp2(1,3) = x10
          sp2(2,0) = co(0)*(w2(0)*x17 - x6)
          sp2(2,1) = co(0)*(cima*x0 + w2(0)*(wl(0,j) - wl(3,j)) + x18)
          sp2(2,2) = co(1)*(w2(2)*x17 + x14)
          sp2(2,3) = -co(1)*(w2(3)*(wl(1,j) - x16) + x9)
          sp2(3,0) = x4
          sp2(3,1) = x8
          sp2(3,2) = x10
          sp2(3,3) = -co(1)*(x12 + x20)
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      case(VFFa1)
!       GammaM(1,3,2)*c0
        pl(:) = PL1(:) + PL2(:)
        x9 = pl(1)*w2(3)
        x10 = pl(2)*w2(3)
        do j = riMaxIn, 0, -1
          x0 = w2(2)*(wl(0,j) + wl(3,j))
          x1 = co(0)*(cima*w2(3)*wl(1,j) + cima*x0 + w2(3)*wl(2,j))
          x2 = -w2(2)*wl(2,j)
          x3 = w2(2)*wl(1,j)
          x4 = w2(3)*(wl(0,j) - wl(3,j))
          x5 = co(0)*(cima*x3 + cima*x4 + x2)
          x6 = cima*wl(2,j)
          x7 = x3 + x4
          x8 = co(0)*(cima*x7 + x2)
          wp(0,j) = -m*x1
          wp(1,j) = -m*x8
          wp(2,j) = co(0)*(cima*pl(1)*x0 - cima*pl(3)*x7 + cima*wl(1,j)*x9 + pl&
                    (3)*w2(2)*wl(2,j) + wl(2,j)*x9)
          wp(3,j) = -co(0)*(-cima*pl(0)*x7 + cima*pl(2)*x0 + cima*wl(1,j)*x10  &
                     + pl(0)*w2(2)*wl(2,j) + wl(2,j)*x10)
          if(rankInc .lt. 1) cycle
          sp2(0,0) = 0
          sp2(0,1) = 0
          sp2(0,2) = x1
          sp2(0,3) = x5
          sp2(1,0) = 0
          sp2(1,1) = 0
          sp2(1,2) = x5
          sp2(1,3) = x1
          sp2(2,0) = 0
          sp2(2,1) = 0
          sp2(2,2) = co(0)*(w2(2)*(wl(1,j) + x6) + x4)
          sp2(2,3) = -co(0)*(w2(3)*(wl(1,j) - x6) + x0)
          sp2(3,0) = 0
          sp2(3,1) = 0
          sp2(3,2) = x1
          sp2(3,3) = -x8
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      end select 
    end subroutine loopVFFa
end module lcVFFa
